\begin{lstlisting}
function [fem0,c]=coupledmodel(kortdata,abvector,hvector)
%Written by Ask H. Larsen, Martin F. Laursen and Kasper Reck 2006
%
%Solves Helmholtz's equation outside a domain defined by the argument. If the argument 'kortdata' is not given, a circle of radius 1 is used. 'abvector' defines the a and b in the rho function and 'hvector' the start and end depth as [a b] and [hstart hend] respectively. To reduce computationtime, terms in the eta and deta function may be removed, at the cost of accuracy. The picturfile should be black & white png format, but other formats may work too.
%
%Example: 'coupledmodel('iceland.png',[2.5 3.5],[0.2 5])'
%
%This example reads the file iceland.png and creates a geometry based on the information contained in this file. Furthermore it sets the ocean depth to vary between 0.2 and 5 in the interval 2.5 to 3.5. The geometry, mesh and solution is plotted.

flclear fem
%Start timer
tic
% COMSOL version
clear vrsn
vrsn.name = 'COMSOL 3.2';
vrsn.ext = 'a';
vrsn.major = 0;
vrsn.build = 300;
vrsn.rcs = '$Name:  $';
vrsn.date = '$Date: 2005/12/20 19:02:30 $';
fem.version = vrsn;

% Geometry
disp('Converting visual data...')
clear draw
g1=circ2(10);
if nargin==3
kort=imread(kortdata);
kort=fliplr(rot90(rot90(kort(:,:,1))));
figure
image(kort);
colormap gray
[c,lr]=flim2curve(kort,{[],[0 255]}); %resolution is 255
%c=mirror(c,[0 1],[0 1]);
disp('Preparing geometry...')
c=solid2(c);
nvertices=geom1get(c,'nv')
c=scale(move(c,-0.5*length(kort),-0.5*length(kort)),1.5/length(kort),1.5/length(kort)); %translate and scale
g4=g1-c;
else
g4=g1-circ2(1);
end
geomplot(g4,'Pointmode','off','edgelabels','on')
draw.s.objs = {g4};
draw.s.name = {'CO1'};
draw.s.tags = {'g4'};
fem.draw = draw;
fem.geom = geomcsg(fem);

% Constants
fem.const = {'a',abvector(1),'b',abvector(2),'g',9.82,'T',1,'hdiff',hvector(2)-hvector(1),'hend',hvector(2)};

%Global expressions
fem.globalexpr={'cinfinity','cinfinity(b^2-(x^2+y^2))./(cinfinity(b^2-(x^2+y^2))+cinfinity((x^2+y^2)-a^2))'}; %
% Initialize mesh
fem.mesh=meshinit(fem);
%Refine mesh
fem.mesh=meshrefine(fem, ...
                    'mcase',0, ...
                    'rmethod','regular');
fem.mesh=meshrefine(fem, ...
                    'mcase',0, ...
                    'rmethod','regular');
%fem.mesh=meshrefine(fem, ...
%                    'mcase',0, ...
%                    'rmethod','regular');
hold on
meshplot(fem.mesh)
fem.mesh		    
% (Default values are not included)

% Application mode 1
clear appl
appl.mode.class = 'Helmholtz';
appl.assignsuffix = '_hzeq';
clear bnd
bnd.type = 'neu';

if nargin==3
bnd.ind = [ones(1,nvertices+4)];
else
bnd.ind = [1,1,1,1,1,1,1,1];
end
appl.bnd = bnd;
clear equ
equ.a = '(2*pi)^2/(g*T^2)';
equ.c = '-hxy';
equ.f = 0;
equ.ind = [1];
appl.equ = equ;
fem.appl{1} = appl;

% Application mode 2
clear appl
appl.mode.class = 'FlPDEWBoundary';
appl.dim = {'lm','lm_t'};
appl.assignsuffix = '_wb';
clear prop
clear weakconstr
weakconstr.value = 'off';
weakconstr.dim = {'lm2','lm3'};
prop.weakconstr = weakconstr;
appl.prop = prop;
clear bnd
bnd.weak = {'+(lm_test*(u-eta)+lm*(u_test-eta_test))-eta_test*deta',0};
bnd.usage = {1,0};
if nargin==3
bnd.ind = [1,1,1,1,2*ones(1,nvertices)];
else
bnd.ind = [1,1,2,2,1,2,2,1];
end
appl.bnd = bnd;
fem.appl{2} = appl;
fem.frame = {'ref'};
fem.border = 1;
fem.units = 'SI';
disp('Analyzing mathematical expressions...')
% Scalar expressions
fem.expr = {'r','sqrt(x^2+y^2)', ...
  'theta','atan2(y,x)', ... 
  'hxy','hend-hdiff*cinfinity',...
  'k','2*pi/(sqrt(g*hxy)*T)', ...
   'eta',    'g0*etacos(0,k,theta,r)+g1*etacos(1,k,theta,r)+g2*etacos(2,k,theta,r)+g3*etacos(3,k,theta,r)+g4*etacos(4,k,theta,r)+g5*etacos(5,k,theta,r)+g6*etacos(6,k,theta,r)+g7*etacos(7,k,theta,r)+g8*etacos(8,k,theta,r)+g9*etacos(9,k,theta,r)+g10*etacos(10,k,theta,r)+g11*etacos(11,k,theta,r)+g12*etacos(12,k,theta,r)+g13*etacos(13,k,theta,r)+g14*etacos(14,k,theta,r)+g15*etacos(15,k,theta,r)+g16*etacos(16,k,theta,r)+g17*etacos(17,k,theta,r)+g18*etacos(18,k,theta,r)+g19*etacos(19,k,theta,r)+g20*etacos(20,k,theta,r)+g21*etacos(21,k,theta,r)+g22*etacos(22,k,theta,r)+z1*etasin(1,k,theta,r)+z2*etasin(2,k,theta,r)+z3*etasin(3,k,theta,r)+z4*etasin(4,k,theta,r)+z5*etasin(5,k,theta,r)+z6*etasin(6,k,theta,r)+z7*etasin(7,k,theta,r)+z8*etasin(8,k,theta,r)+z9*etasin(9,k,theta,r)+z10*etasin(10,k,theta,r)+z11*etasin(11,k,theta,r)+z12*etasin(12,k,theta,r)+z13*etasin(13,k,theta,r)+z14*etasin(14,k,theta,r)+z15*etasin(15,k,theta,r)+z16*etasin(16,k,theta,r)+z17*etasin(17,k,theta,r)+z18*etasin(18,k,theta,r)+z19*etasin(19,k,theta,r)+z20*etasin(20,k,theta,r)+z21*etasin(21,k,theta,r)+z22*etasin(22,k,theta,r)+exp(i*k*x)', ...  
'deta','g0*diff(etacos(0,k,theta,r),r)+g1*diff(etacos(1,k,theta,r),r)+g2*diff(etacos(2,k,theta,r),r)+g3*diff(etacos(3,k,theta,r),r)+g4*diff(etacos(4,k,theta,r),r)+g5*diff(etacos(5,k,theta,r),r)+g6*diff(etacos(6,k,theta,r),r)+g7*diff(etacos(7,k,theta,r),r)+g8*diff(etacos(8,k,theta,r),r)+g9*diff(etacos(9,k,theta,r),r)+g10*diff(etacos(10,k,theta,r),r)+g11*diff(etacos(11,k,theta,r),r)+g12*diff(etacos(12,k,theta,r),r)+g13*diff(etacos(13,k,theta,r),r)+g14*diff(etacos(14,k,theta,r),r)+g15*diff(etacos(15,k,theta,r),r)+g16*diff(etacos(16,k,theta,r),r)+g17*diff(etacos(17,k,theta,r),r)+g18*diff(etacos(18,k,theta,r),r)+g19*diff(etacos(19,k,theta,r),r)+g20*diff(etacos(20,k,theta,r),r)+g21*diff(etacos(21,k,theta,r),r)+g22*diff(etacos(22,k,theta,r),r)+z1*diff(etasin(1,k,theta,r),r)+z2*diff(etasin(2,k,theta,r),r)+z3*diff(etasin(3,k,theta,r),r)+z4*diff(etasin(4,k,theta,r),r)+z5*diff(etasin(5,k,theta,r),r)+z6*diff(etasin(6,k,theta,r),r)+z7*diff(etasin(7,k,theta,r),r)+z8*diff(etasin(8,k,theta,r),r)+z9*diff(etasin(9,k,theta,r),r)+z10*diff(etasin(10,k,theta,r),r)+z11*diff(etasin(11,k,theta,r),r)+z12*diff(etasin(12,k,theta,r),r)+z13*diff(etasin(13,k,theta,r),r)+z14*diff(etasin(14,k,theta,r),r)+z15*diff(etasin(15,k,theta,r),r)+z16*diff(etasin(16,k,theta,r),r)+z17*diff(etasin(17,k,theta,r),r)+z18*diff(etasin(18,k,theta,r),r)+z19*diff(etasin(19,k,theta,r),r)+z20*diff(etasin(20,k,theta,r),r)+z21*diff(etasin(21,k,theta,r),r)+z22*diff(etasin(22,k,theta,r),r)+i*k*cos(theta)*exp(i*k*x)'};

% Functions
clear fcns
%cosine part
fcns{1}.type='inline';
fcns{1}.name='etacos(n,k,theta,r)';
fcns{1}.expr='besselh(n,1,k*r)*cos(n*theta)';
fcns{1}.dexpr={'0','0','0', ...
  '(-k*besselh(n+1,1,k*r)+n/r*besselh(n,1,k*r))*cos(n*theta)'};
fcns{1}.complex='true';

%sine part
fcns{2}.type='inline';
fcns{2}.name='etasin(n,k,theta,r)';
fcns{2}.expr='besselh(n,1,k*r)*sin(n*theta)';
fcns{2}.dexpr={'0','0','0', ...
  '(-k*besselh(n+1,1,k*r)+n/r*besselh(n,1,k*r))*sin(n*theta)'};
fcns{2}.complex='true';

fem.functions = fcns;

% ODE Settings
clear ode
ode.dim={'g0','g1','g2','g3','g4','g5','g6','g7','g8','g9','g10','g11','g12','g13','g14','g15','g16','g17','g18','g19','g20','g21','g22','z1','z2','z3','z4','z5','z6','z7','z8','z9','z10','z11','z12','z13','z14','z15','z16','z17','z18','z19','z20','z21','z22'};
ode.f={'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};
ode.init={'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};
ode.dinit={'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};
fem.ode=ode;
% Multiphysics
fem=multiphysics(fem);

% Extend mesh
fem.xmesh=meshextend(fem);

% Solve problem
disp('Solving complex differential equations...Please wait')
fem.sol=femlin(fem, ...
               'complexfun','on', ...
               'solcomp',{'g0','g1','g2','g3','g4','g5','g6','g7','g8','g9','g10','g11','g12','g13','g14','g15','g16','g17','g18','g19','g20','g21','g22','z1','z2','z3','z4','z5','z6','z7','z8','z9','z10','z11','z12','z13','z14','z15','z16','z17','z18','z19','z20','z21','z22','lm','u'}, ...
               'outcomp',{'g0','g1','g2','g3','g4','g5','g6','g7','g8','g9','g10','g11','g12','g13','g14','g15','g16','g17','g18','g19','g20','g21','g22','z1','z2','z3','z4','z5','z6','z7','z8','z9','z10','z11','z12','z13','z14','z15','z16','z17','z18','z19','z20','z21','z22','lm','u'});

% Save current fem structure for restart purposes
fem0=fem;

% Plot solution
figure
postplot(fem, ...
         'tridata',{'abs(u)','cont','internal'}, ...
         'trimap','jet(1024)', ...
         'title','Surface: u', ...
         'axis',[-5.7956416464891,5.7956416464891,-2.2,2.2,-1,1]);
%Save solution
u0=fem.sol.u(:);

disp('Extracting meaningless coefficients...')
%Initialize the system with zeroes except the ODE variables
fem.equ.init={'0'};
fem.ode.init={'1','2','3','4','5','6','7','8','9','10','11','12','13','14','15','16','17','18','19','20','21','22','23','24','25','26','27','28','29','30','31','32','33','34','35','36','37','38','39','40','41','42','43','44','45'};

%Extend mesh
fem.xmesh=meshextend(fem);

%Initialize the solution vector
fem.sol=asseminit(fem,'out','u');

% Locate the position of the ODE variables and display their value.
disp('Numerical G_n from Comsol')
for i = 1:length(fem.ode.init)
eta_n_index(i) = find(fem.sol == i);
disp(['coefficient' num2str(i-1) ': ' num2str(u0(eta_n_index(i))) ...
' ,abs: ' num2str(abs(u0(eta_n_index(i))))])
end
% Insert the true solution back into the fem struct
fem.sol = asseminit(fem,'init',u0);

%disp('Computing ground zero...')
%j=1;
%for i=5:nvertices-4
%    j=j+1
%    I(j)=postint(fem,'u', ...
%           'dl',[i], ...
%           'edim',1);
%end
%[e,k]=max(I)
%disp(['Casualties: '  num2str(round(abs(e)*1000000))])
toc
\end{lstlisting}
